@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2017 by Andrew Mustun. All rights reserved.
+ * Copyright (c) 2011-2018 by Andrew Mustun. All rights reserved.
  * 
  * This file is part of the QCAD project.
  *
@@ -424,6 +424,7 @@ ShapeAlgorithms.autoSplit = function(shape, otherShapes, position, extend) {
     }
 
     // find intersection points closest to position:
+    // array of two distances and two point vectors:
     var cutDistances = ShapeAlgorithms.getClosestIntersectionPointDistances(shape, ips, position);
 
     // distance along shape to clicked position:
@@ -452,10 +453,23 @@ ShapeAlgorithms.autoSplit = function(shape, otherShapes, position, extend) {
         cutPos2 = undefined;
     }
 
-    return ShapeAlgorithms.autoSplitManual(shape, cutDist1, cutDist2, cutPos1, cutPos2, position);
+    return ShapeAlgorithms.autoSplitManual(shape, cutDist1, cutDist2, cutPos1, cutPos2, position, extend);
 };
 
-ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, cutPos2, position) {
+/**
+ * Cut shape at given distances / positions.
+ *
+ * \return Array of three new shapes which each might be undefined if its
+ * length would otherwise be 0.
+ * The first shape is the rest at the start of the shape.
+ * The second shape is the rest at the end of the shape.
+ * The third shape is the segment self in its new shape.
+ */
+ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, cutPos2, position, extend) {
+    if (isNull(extend)) {
+        extend = false;
+    }
+
 //    if (!isCircleShape(shape) && !isFullEllipseShape(shape) &&
 //        !isXLineShape(shape) && !isRayShape(shape)) {
 
@@ -485,10 +499,18 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
         cutDist2 = shape.getDistanceFromStart(cutPos2);
     }
 
-    if (RMath.fuzzyCompare(cutDist1, 0.0) && shape.getStartPoint().equalsFuzzy(cutPos1) &&
-        RMath.fuzzyCompare(cutDist2, shape.getLength()) && shape.getEndPoint().equalsFuzzy(cutPos2)) {
+    if (isNull(cutDist2)) {
+        // ray, nothing to cut:
+        if (RMath.fuzzyCompare(cutDist1, 0.0) && shape.getStartPoint().equalsFuzzy(cutPos1)) {
+            return [undefined, undefined, shape.clone()];
+        }
+    }
+    else {
+        if (RMath.fuzzyCompare(cutDist1, 0.0) && shape.getStartPoint().equalsFuzzy(cutPos1) &&
+            RMath.fuzzyCompare(cutDist2, shape.getLength()) && shape.getEndPoint().equalsFuzzy(cutPos2)) {
 
-        return [undefined, undefined, shape.clone()];
+            return [undefined, undefined, shape.clone()];
+        }
     }
 
     var rest1 = undefined;
@@ -589,6 +611,7 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
 //            cutDist2 = undefined;
 //        }
 
+
         // <--------x-------o-------x---------
         // rest2   cp2   segment   cp1   rest1
         if (isValidVector(cutPos1) && isValidVector(cutPos2)) {
@@ -624,31 +647,33 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
         rest2.trimStartPoint(cutDist2);
 
         segment = shape.clone();
-        var l1 = segment.getLength();
-        segment.trimStartPoint(cutDist1);
-        var l2 = segment.getLength();
-        segment.trimEndPoint(cutDist2 - (l1-l2));
+        //var l1 = segment.getLength();
+        segment.setStartAngle(segment.getCenter().getAngleTo(cutPos1));
+        //segment.trimStartPoint(cutDist1);
+        //var l2 = segment.getLength();
+        //segment.trimEndPoint(cutDist2 - (l1-l2));
+        segment.setEndAngle(segment.getCenter().getAngleTo(cutPos2));
 
-        var angleLength1 = rest1.getAngleLength(true);
-        var angleLength2 = rest2.getAngleLength(true);
+        if (!extend) {
+            var angleLength1 = rest1.getAngleLength(true);
+            var angleLength2 = rest2.getAngleLength(true);
+            if (angleLength1+angleLength2 > shape.getAngleLength()) {
+                rest1.trimEndPoint(cutDist2);
+                rest2.trimStartPoint(cutDist1);
 
-        if (angleLength1+angleLength2 > shape.getAngleLength()) {
-            rest1.trimEndPoint(cutDist2);
-            rest2.trimStartPoint(cutDist1);
+                segment.trimStartPoint(cutDist2);
+                segment.trimEndPoint(cutDist1);
 
-            segment.trimStartPoint(cutDist2);
-            segment.trimEndPoint(cutDist1);
+                angleLength1 = rest1.getAngleLength(true);
+                angleLength2 = rest2.getAngleLength(true);
+            }
+            if (angleLength1<1.0e-5) {
+                rest1 = undefined;
+            }
 
-            angleLength1 = rest1.getAngleLength(true);
-            angleLength2 = rest2.getAngleLength(true);
-        }
-
-        if (angleLength1<1.0e-5) {
-            rest1 = undefined;
-        }
-
-        if (angleLength2<1.0e-5) {
-            rest2 = undefined;
+            if (angleLength2<1.0e-5) {
+                rest2 = undefined;
+            }
         }
     }
 
@@ -952,7 +977,8 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
 
 /**
  * \return The two distances along the given shape identifying the
- * intersections points closest to the given position.
+ * intersections points closest to the given position along with the cut positions:
+ * [ [cutDist1, cutDist2], [cutPos1, cutPos2] ]
  *
  * \param onShape True: only return intersections on the shape
  * (for trimming, breaking, default).
@@ -1022,6 +1048,10 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
     if (isPolylineShape(shape) && shape.isGeometricallyClosed()) {
         circular = true;
     }
+    if (isArcShape(shape)) {
+        circular = true;
+    }
+
 //    if (isSplineShape(shape) && shape.isGeometricallyClosed()) {
 //        circular = true;
 //    }
@@ -1046,7 +1076,7 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
     var cutPos1 = undefined;
     var cutPos2 = undefined;
 
-    // for circular shapes, also find intersections closest to start point:
+    // for circular shapes, also find intersections closest to and furthest from start point:
     var cutDistMax = undefined;
     var cutDistMin = undefined;
     var cutPosMax = undefined;
@@ -1057,11 +1087,11 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
 
         if (isEllipseShape(shape)) {
             dist = RMath.getAngleDifference(orthoLine.getAngle(), shape.getCenter().getAngleTo(ip));
-            if (isNull(cutDist1) || dist>cutDist1) {
+            if (isNull(cutDist1) || dist<cutDist1) {
                 cutPos1 = ip;
                 cutDist1 = dist;
             }
-            if (isNull(cutDist2) || dist<cutDist2) {
+            if (isNull(cutDist2) || dist>cutDist2) {
                 cutPos2 = ip;
                 cutDist2 = dist;
             }
@@ -1097,7 +1127,7 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
                 }
 
                 if (circular) {
-                    if (isNull(cutDistMin) || dist<cutDistMin) {
+                    if (isNull(cutDistMin) || (dist<cutDistMin && (dist>pDist || isPolylineShape(shape)))) {
                         cutDistMin = dist;
                         cutPosMin = ip;
                     }
@@ -1106,6 +1136,27 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
         }
     }
 
+    if (circular) {
+        if (isNull(cutDist1)) {
+            cutDist1 = cutDistMax;
+            cutPos1 = cutPosMax;
+        }
+        if (isNull(cutDist2)) {
+            cutDist2 = cutDistMin;
+            cutPos2 = cutPosMin;
+        }
+    }
+
+    if (isEllipseShape(shape) && shape.isReversed()) {
+        var dummy = cutPos1;
+        cutPos1 = cutPos2;
+        cutPos2 = dummy;
+
+        dummy = cutDist1;
+        cutDist1 = cutDist2;
+        cutDist2 = dummy;
+    }
+
     // open shape: cut to start or end point:
     if (!isCircleShape(shape) &&
         !isFullEllipseShape(shape) &&
@@ -1128,17 +1179,6 @@ ShapeAlgorithms.getClosestIntersectionPointDistances = function(shape, intersect
         shape.reverse();
     }
 
-    if (circular) {
-        if (isNull(cutDist1)) {
-            cutDist1 = cutDistMax;
-            cutPos1 = cutPosMax;
-        }
-        if (isNull(cutDist2)) {
-            cutDist2 = cutDistMin;
-            cutPos2 = cutPosMin;
-        }
-    }
-
     return [ [cutDist1, cutDist2], [cutPos1, cutPos2] ];
 
     /*
@@ -1624,157 +1664,16 @@ ShapeAlgorithms.createEllipseInscribedFromLines = function(line1, line2, line3,
     if (isNull(quad)) {
         return undefined;
     }
-    return ShapeAlgorithms.createEllipseInscribedFromVertices(quad[0], quad[1], quad[2], quad[3]);
+    return REllipse.createInscribed(quad[0], quad[1], quad[2], quad[3]);
 };
 
 
 /**
  * Produces an ellipse inscribed in the quadrilateral defined by the
  * four given ordered vertices (RVector).
- *
- * Based on:
- * http://chrisjones.id.au/Ellipses/ellipse.html
- * http://mathworld.wolfram.com/Ellipse.html
  */
 ShapeAlgorithms.createEllipseInscribedFromVertices = function(v1, v2, v3, v4) {
-    var scale = undefined;
-    var offset = new RVector(0,0);
-
-    var i;
-    var quad = [v1, v2, v3, v4];
-
-    var extr = new RBox(RVector.getMinimum(quad), RVector.getMaximum(quad));
-
-    // offset to reduce magnitudes:
-    offset = extr.getCenter().getNegated();
-    v1.move(offset);
-    v2.move(offset);
-    v3.move(offset);
-    v4.move(offset);
-
-    // numbers in this algorithm can get extremely large, so we scale down here
-    // a bit if appropriate:
-    if (extr.getWidth()>100.0 || extr.getHeight()>100.0) {
-        scale = 100.0/Math.max(extr.getWidth(), extr.getHeight());
-        v1.scale(scale);
-        v2.scale(scale);
-        v3.scale(scale);
-        v4.scale(scale);
-    }
-
-    var edges = [];
-    for (i=0; i<4; i++) {
-        edges[i] = new RLine(quad[i], quad[(i+1)%4]);
-    }
-
-    var x0 = quad[0].x;
-    var y0 = quad[0].y;
-    var x1 = quad[1].x;
-    var y1 = quad[1].y;
-    var x2 = quad[2].x;
-    var y2 = quad[2].y;
-    var x3 = quad[3].x;
-    var y3 = quad[3].y;
-
-    var ma =  x1 * x2 * y3 - x0 * x2 * y3 - x1 * y2 * x3 + x0 * y2 * x3 - x0 * y1 * x3 + y0 * x1 * x3 + x0 * y1 * x2 - y0 * x1 * x2;
-    var mb =  x0 * x2 * y3 - x0 * x1 * y3 - x1 * y2 * x3 + y1 * x2 * x3 - y0 * x2 * x3 + y0 * x1 * x3 + x0 * x1 * y2 - x0 * y1 * x2;
-    var mc =  x1 * x2 * y3 - x0 * x1 * y3 - x0 * y2 * x3 - y1 * x2 * x3 + y0 * x2 * x3 + x0 * y1 * x3 + x0 * x1 * y2 - y0 * x1 * x2;
-    var md =  y1 * x2 * y3 - y0 * x2 * y3 - x0 * y1 * y3 + y0 * x1 * y3 - y1 * y2 * x3 + y0 * y2 * x3 + x0 * y1 * y2 - y0 * x1 * y2;
-    var me = -x1 * y2 * y3 + x0 * y2 * y3 + y1 * x2 * y3 - x0 * y1 * y3 - y0 * y2 * x3 + y0 * y1 * x3 + y0 * x1 * y2 - y0 * y1 * x2;
-    var mf =  x1 * y2 * y3 - x0 * y2 * y3 + y0 * x2 * y3 - y0 * x1 * y3 - y1 * y2 * x3 + y0 * y1 * x3 + x0 * y1 * y2 - y0 * y1 * x2;
-    var mg =  x1 * y3 - x0 * y3 - y1 * x3 + y0 * x3 - x1 * y2 + x0 * y2 + y1 * x2 - y0 * x2;
-    var mh =  x2 * y3 - x1 * y3 - y2 * x3 + y1 * x3 + x0 * y2 - y0 * x2 - x0 * y1 + y0 * x1;
-    var mi =  x2 * y3 - x0 * y3 - y2 * x3 + y0 * x3 + x1 * y2 - y1 * x2 + x0 * y1 - y0 * x1;
-
-    var T = RMatrix.create3x3(
-        ma, mb, mc,
-        md, me, mf,
-        mg, mh, mi
-    );
-
-    var TI = T.getInverse();
-
-    var mj = TI.get(0, 0);
-    var mk = TI.get(0, 1);
-    var ml = TI.get(0, 2);
-    var mm = TI.get(1, 0);
-    var mn = TI.get(1, 1);
-    var mo = TI.get(1, 2);
-    var mp = TI.get(2, 0);
-    var mq = TI.get(2, 1);
-    var mr = TI.get(2, 2);
-
-    var a = mj*mj + mm*mm - mp*mp;
-    var b = mj*mk + mm*mn - mp*mq;
-    var c = mk*mk + mn*mn - mq*mq;
-    var d = mj*ml + mm*mo - mp*mr;
-    var f = mk*ml + mn*mo - mq*mr;
-    var g = ml*ml + mo*mo - mr*mr;
-
-    var term = (b*b - a*c);
-
-    var cx, cy, axis1, axis2;
-
-    // circle:
-    if (Math.abs(term)<1e-50) {
-        cx = (v1.x + v3.x)/2;
-        cy = (v1.y + v3.y)/2;
-        axis1 = v1.getDistanceTo(v2)/2;
-        axis2 = axis1;
-    }
-    else {
-        cx = (c*d - b*f) / term;
-        cy = (a*f - b*d) / term;
-
-        var term1 = (2 * (a*f*f + c*d*d + g*b*b - 2*b*d*f - a*c*g));
-        var term2 = Math.sqrt((a-c)*(a-c) + 4*b*b);
-        var term3 = (a + c);
-
-        axis1 = Math.sqrt(term1 / (term * (term2-term3)));
-        axis2 = Math.sqrt(term1 / (term * (-term2-term3)));
-    }
-
-    var center = new RVector(cx, cy);
-
-    var angle;
-
-    if (b===0.0) {
-        angle = 0;
-    }
-    else {
-        var arccot = function(v) {
-            return Math.PI/2 - Math.atan(v);
-        };
-
-        angle = 0.5 * arccot((a-c)/(2*b));
-    }
-
-    if (a>c) {
-        angle += Math.PI/2;
-    }
-
-    var majorPoint = RVector.createPolar(axis1, angle);
-
-    var ellipse = new REllipse(center, majorPoint, axis2/axis1, 0.0, 2*Math.PI, false);
-
-    // workaround if the algorithm above does not produce the correct angle
-    // (90 degrees off):
-    for (i=0; i<4; i++) {
-        var ips = ellipse.getIntersectionPoints(edges[i], false);
-        if (ips.length===2) {
-            if (!ips[0].equalsFuzzy(ips[1], 0.001)) {
-                ellipse.rotate(Math.PI/2, ellipse.getCenter());
-                break;
-            }
-        }
-    }
-
-    if (!isNull(scale)) {
-        ellipse.scale(new RVector(1.0/scale,1.0/scale));
-    }
-    ellipse.move(offset.getNegated());
-
-    return ellipse;
+    return REllipse.createInscribed(v1, v2, v3, v4);
 };
 
 /**
@@ -1803,9 +1702,9 @@ ShapeAlgorithms.splineToLineOrArc = function(spline, tolerance, linesOnly) {
                 }
             }
 
-            if (splineIsArc && arc.getRadius()>1000.0) {
-                return new RLine(startPoint, endPoint);
-            }
+//            if (splineIsArc && arc.getRadius()>1000.0) {
+//                return new RLine(startPoint, endPoint);
+//            }
 
             if (splineIsArc) {
                 return arc;
@@ -1833,11 +1732,7 @@ ShapeAlgorithms.splineToLineOrArc = function(spline, tolerance, linesOnly) {
  * Converts the given circle into an arc with the given start angle or 0.
  */
 ShapeAlgorithms.circleToArc = function(circle, startAngle) {
-    if (isNull(startAngle)) {
-        startAngle = 0.0;
-    }
-
-    return new RArc(circle.getCenter(), circle.getRadius(), startAngle, startAngle + 2*Math.PI, false);
+    return circle.toArc(startAngle);
 };
 
 /**
@@ -1912,24 +1807,7 @@ ShapeAlgorithms.splitAt = function(shape, points) {
  * the given ellipse best.
  */
 ShapeAlgorithms.ellipseToArcCircleEllipse = function(ellipse) {
-    if (ellipse.isCircular()) {
-        if (ellipse.isFullEllipse()) {
-            return new RCircle(ellipse.getCenter(), ellipse.getMajorRadius());
-        }
-        else {
-            var c = ellipse.getCenter();
-            var ret = new RArc(c, ellipse.getMajorRadius(),
-                //ellipse.getStartAngle(), ellipse.getEndAngle(),
-                0.0, 2*Math.PI,
-                ellipse.isReversed());
-            ret.setStartAngle(c.getAngleTo(ellipse.getStartPoint()));
-            ret.setEndAngle(c.getAngleTo(ellipse.getEndPoint()));
-            return ret;
-        }
-    }
-    else {
-        return ellipse;
-    }
+    return RShape.ellipseToArcCircleEllipse(ellipse);
 };
 
 /**
@@ -1937,6 +1815,8 @@ ShapeAlgorithms.ellipseToArcCircleEllipse = function(ellipse) {
  * using the given function which projects an RVector to another RVector.
  *
  * \returns An REllipse or in special cases an RArc or RCircle.
+ *
+ * OBSOLETE use RShape::transformArc instead
  */
 ShapeAlgorithms.transformArc = function(arc, fun) {
     var r1, r2;
@@ -1950,17 +1830,20 @@ ShapeAlgorithms.transformArc = function(arc, fun) {
         r2 = new RVector(0, arc.getRadius());
     }
 
+    // get vertices at extremities:
     var v1 = arc.getCenter().operator_add(r1).operator_add(r2);
     var v2 = arc.getCenter().operator_add(r1).operator_subtract(r2);
     var v3 = arc.getCenter().operator_subtract(r1).operator_subtract(r2);
     var v4 = arc.getCenter().operator_subtract(r1).operator_add(r2);
 
+    // project those vertices:
     fun(v1);
     fun(v2);
     fun(v3);
     fun(v4);
 
     //var ret = [];
+    // inscribe ellipse into vertices:
     var ellipse = ShapeAlgorithms.createEllipseInscribedFromVertices(v1, v2, v3, v4);
     //ret.push(ellipse.copy());
 
@@ -2133,7 +2016,13 @@ ShapeAlgorithms.divideShape = function(shape, pos1, pos2) {
 
         shape1 = trimEndPoint(shape1, pos1, pos1);
 
-        cutPos1 = shape1.getEndPoint();
+        if (isRayShape(shape1)) {
+            // ray points in opposite direction:
+            cutPos1 = shape1.getStartPoint();
+        }
+        else {
+            cutPos1 = shape1.getEndPoint();
+        }
 
         shape2 = trimStartPoint(shape2, pos1, pos1);
 
