@@ -23,6 +23,15 @@ include("scripts/File/OpenFile/OpenFile.js");
 include("scripts/File/AutoSave/AutoSave.js");
 include("scripts/Tools/arguments.js");
 
+// allow plugins to extend autostart through autostart1.js, autostart2.js, ...:
+for (var i=1; i<10; i++) {
+    var fn = "scripts/autostart" + i + ".js";
+    if (new QFileInfo(fn).exists() || new QFileInfo(":/" + fn).exists()) {
+        include(fn);
+    }
+}
+
+
 /**
  * Prints version information.
  */
@@ -63,8 +72,8 @@ function usage() {
           + "                                 after staring QCAD. Options after the script\n"
           + "                                 file are passed on to the script.\n"
           + "-help                            Displays this help.\n"
-          + "-ignore-script-files             Ignore script files on disk."
-          + "                                 Only load scripts from plugins if applicable."
+          + "-ignore-script-files             Ignore script files on disk.\n"
+          + "                                 Only load scripts from plugins if applicable.\n"
           + "-locale [locale]                 Sets the locale to be used (overrides\n"
           + "                                 the language set in the preferences).\n"
           + "                                 E.g. '-locale de' starts QCAD in German.\n"
@@ -594,9 +603,14 @@ function main() {
     // theme:
     applyTheme();
 
+    // native / non-native menubar:
+    if (RSettings.getBoolValue("MenuBar/UseNativeMenuBar", true)===false) {
+        QCoreApplication.setAttribute(Qt.AA_DontUseNativeMenuBar);
+    }
+
     // splash:
     var splash = undefined;
-    if (RSettings.getBoolValue("Start/EnableSplashScreen", true)) {
+    if (RSettings.getBoolValue("Startup/EnableSplashScreen", true)) {
         var fn;
         var key;
         if (RSettings.getDevicePixelRatio()===2) {
@@ -793,7 +807,11 @@ function main() {
     var clickedFilesAndArgs = filesToOpen.concat(args.slice(1));
     QCoreApplication.processEvents();
     appWin.setProperty("starting", true);
-    openFiles(clickedFilesAndArgs, !recovered);
+    var restored = false;
+    if (typeof(restoreFiles)=="function") {
+        restored = restoreFiles();
+    }
+    openFiles(clickedFilesAndArgs, !recovered && !restored);
     appWin.setProperty("starting", false);
 
     RPluginLoader.postInitPlugins(RPluginInterface.LoadedFiles);
